import type { TableData, BaseFieldInfo } from '@/types/Tables/table'
import type { BaseTableInfo } from '@/types/Tables/tablePageData'
import type { ResponseInfo } from '@/types/axios'
import type {
  BaseFieldItem,
  TableFields,
  TableInfoItem,
  SaveFields,
} from '@/types/Tables/table'

import axiosInstance from '@/utils/axios/axiosInstance'
export function useTable() {
  const getData = async (
    url: string,
    config: Object = {},
  ): Promise<Array<TableData>> => {
    try {
      const res = (await axiosInstance.get(url, config)) as ResponseInfo

      if (res.code !== 0) throw new Error('请求失败')
      return res.data as TableData[]
    } catch {
      console.log('数据请求失败')
      return []
    }
  }

  /**
   * @description: 判断当前字段是否是需要固定的字段
   * @param {*} info 字段信息
   * @return {boolean}
   */
  const isFixedField = (
    fixedFields: Array<string>,
    info: BaseFieldInfo,
  ): boolean => {
    return fixedFields.includes(info.name)
  }

  /**
   * @description: 保存表格信息
   * @return {*}
   */
  const saveTableInfo = (
    name: string,
    tableInfo: TableInfoItem<TableData, BaseFieldItem<TableFields>>,
  ) => {
    try {
      let result = {
        fields: tableInfo.fields, // 字段数组
        tableSortedFields: tableInfo.tableSortedFields, // 排序过后的表格字段，仅用于展示表格字段
      }
      localStorage.setItem(name, JSON.stringify(result))
    } catch (err) {
      console.log('保存失败')
      console.log(err)
    }
  }

  /**
   * @description: 获取保存的表格信息
   * @param {*} TableInfo
   * @return {*}
   */
  const getTableInfo = (name: string): SaveFields | undefined => {
    try {
      let info = localStorage.getItem(name)
      if (!info) throw new Error('没有该信息')
      return JSON.parse(info) as {
        fields: BaseFieldItem<TableFields>[]
        tableSortedFields: TableFields[]
      }
    } catch (err) {
      console.log('获取失败')
      console.log(err)
      return undefined
    }
  }

  /**
   * @description: 更新自定义指标和表格字段
   * @param {*} newIndicator  新的指标信息
   * @param {*} newSortedTablefileds 新的表格字段信息
   * @return {*}
   */
  const updateTableFields = (
    tableInfo: BaseTableInfo,
    activeName: string,
    newTableFiles: Array<BaseFieldItem<TableFields>>,
    newSortedTablefileds: Array<TableFields>,
  ): Promise<boolean> => {
    return new Promise((reslove, reject) => {
      try {
        let oldTablefields = tableInfo[activeName].fields
        let oldSortedFields = tableInfo[activeName].tableSortedFields

        oldTablefields.splice(0, oldTablefields.length, ...newTableFiles)

        oldSortedFields.splice(
          0,
          oldSortedFields.length,
          ...newSortedTablefileds,
        )
        reslove(true)
      } catch (err) {
        console.log('更新失败')
        console.log(err)
        reject(false)
      }
    })
  }

  return {
    getData,
    isFixedField,
    saveTableInfo,
    getTableInfo,
    updateTableFields,
  }
}
